@@ -82,6 +82,7 @@ from pyteal.ast.abi.util import (
     make,
     size_of,
     type_spec_from_annotation,
+    type_spec_from_algosdk,
     type_specs_from_signature,
     contains_type_spec,
 )
@@ -161,6 +162,7 @@ __all__ = [
     "KeyRegisterTransactionTypeSpec",
     "TransactionTypeSpecs",
     "type_spec_from_annotation",
+    "type_spec_from_algosdk",
     "type_specs_from_signature",
     "make",
     "size_of",
