@@ -499,3 +499,97 @@ def type_specs_from_signature(sig: str) -> tuple[list[TypeSpec], Optional[TypeSp
         return_type = type_spec_from_algosdk(sdk_method.returns.type)
 
     return [type_spec_from_algosdk(arg.type) for arg in sdk_method.args], return_type
+
+
+def type_spec_is_assignable_to(a: TypeSpec, b: TypeSpec) -> bool:
+    """Decides if the value of type :code:`a` can be assigned to or interpreted as another value of type :code:`b`.
+
+    This method return true if and only if all of the following properties hold:
+
+        * value of type :code:`a` has identical encoding as value of type :code:`b`
+        * type :code:`b` is as general as, or more general than type :code:`a`
+
+    For `abi.NamedTuple`, we allow mutual assigning between `abi.Tuple` and `abi.NamedTuple`.
+    But between `abi.NamedTuple`, we only return true when the type specs are identical, or we cannot compare against generality.
+
+    Some examples are illustrated as following:
+
+    =========================== =========================== ============= ====================================================================
+    Type :code:`a`              Type :code:`b`              Assignable?   Reason
+    =========================== =========================== ============= ====================================================================
+    :code:`DynamicArray[Byte]`  :code:`DynamicBytes`        :code:`True`  :code:`DynamicBytes` is as general as :code:`DynamicArray[Byte]`
+    :code:`DynamicBytes`        :code:`DynamicArray[Byte]`  :code:`True`  :code:`DynamicArray[Byte]` is as general as :code:`DynamicBytes`
+    :code:`StaticArray[Byte,N]` :code:`StaticBytes[N]`      :code:`True`  :code:`StaticBytes[N]` is as general as :code:`StaticArray[Byte,N]`
+    :code:`StaticBytes[N]`      :code:`StaticArray[Byte,N]` :code:`True`  :code:`StaticArray[Byte,N]` is as general as :code:`StaticBytes[N]`
+    :code:`String`              :code:`DynamicBytes`        :code:`True`  :code:`DynamicBytes` is more general than :code:`String`
+    :code:`DynamicBytes`        :code:`String`              :code:`False` :code:`String` is more specific than :code:`DynamicBytes`
+    :code:`Address`             :code:`StaticBytes[32]`     :code:`True`  :code:`StaticBytes[32]` is more general than :code:`Address`
+    :code:`StaticBytes[32]`     :code:`Address`             :code:`False` :code:`Address` is more specific than :code:`StaticBytes[32]`
+    :code:`PaymentTransaction`  :code:`Transaction`         :code:`True`  :code:`Transaction` is more general than :code:`PaymentTransaction`
+    :code:`Transaction`         :code:`PaymentTransaction`  :code:`False` :code:`PaymentTransaction` is more specific than :code`Transaction`
+    :code:`Uint8`               :code:`Byte`                :code:`True`  :code:`Uint8` is as general as :code:`Byte`
+    :code:`Byte`                :code:`Uint8`               :code:`True`  :code:`Byte` is as general as :code:`Uint8`
+    =========================== =========================== ============= ====================================================================
+
+    Args:
+        a: The abi.TypeSpec of the value on the right hand side of the assignment.
+        b: The abi.TypeSpec of the value on the left hand side of the assignment.
+
+    Returns:
+        A boolean result on if type :code:`a` is assignable to type :code:`b`.
+    """
+
+    from pyteal.ast.abi import (
+        TupleTypeSpec,
+        NamedTupleTypeSpec,
+        ArrayTypeSpec,
+        StaticArrayTypeSpec,
+        DynamicArrayTypeSpec,
+        StringTypeSpec,
+        AddressTypeSpec,
+        UintTypeSpec,
+    )
+
+    match a, b:
+        case NamedTupleTypeSpec(), NamedTupleTypeSpec():
+            return a == b
+        case TupleTypeSpec(), TupleTypeSpec():
+            a, b = cast(TupleTypeSpec, a), cast(TupleTypeSpec, b)
+            if a.length_static() != b.length_static():
+                return False
+            return all(
+                map(
+                    lambda ab: type_spec_is_assignable_to(ab[0], ab[1]),
+                    zip(a.value_type_specs(), b.value_type_specs()),
+                )
+            )
+        case ArrayTypeSpec(), ArrayTypeSpec():
+            a, b = cast(ArrayTypeSpec, a), cast(ArrayTypeSpec, b)
+            if not type_spec_is_assignable_to(a.value_type_spec(), b.value_type_spec()):
+                return False
+            match a, b:
+                case AddressTypeSpec(), StaticArrayTypeSpec():
+                    a, b = cast(AddressTypeSpec, a), cast(StaticArrayTypeSpec, b)
+                    return a.length_static() == b.length_static()
+                case StaticArrayTypeSpec(), AddressTypeSpec():
+                    return False
+                case StaticArrayTypeSpec(), StaticArrayTypeSpec():
+                    a, b = cast(StaticArrayTypeSpec, a), cast(StaticArrayTypeSpec, b)
+                    return a.length_static() == b.length_static()
+                case StringTypeSpec(), DynamicArrayTypeSpec():
+                    return True
+                case DynamicArrayTypeSpec(), StringTypeSpec():
+                    return False
+                case DynamicArrayTypeSpec(), DynamicArrayTypeSpec():
+                    return True
+            return False
+        case UintTypeSpec(), UintTypeSpec():
+            a, b = cast(UintTypeSpec, a), cast(UintTypeSpec, b)
+            return a.size == b.size
+
+    if isinstance(a, type(b)):
+        return True
+    elif str(a) == str(b):
+        return True
+
+    return False
