@@ -261,6 +261,8 @@ class SubroutineDefinition:
         self,
         args: list[Expr | ScratchVar | abi.BaseType],
     ) -> "SubroutineCall":
+        from pyteal.ast.abi.util import type_spec_is_assignable_to
+
         if len(args) != self.argument_count():
             raise TealInputError(
                 f"Incorrect number of arguments for subroutine call. "
@@ -282,7 +284,8 @@ class SubroutineDefinition:
                     raise TealInputError(
                         f"supplied argument at index {i} should be an ABI type but got {arg}"
                     )
-                if arg.type_spec() != arg_type:
+
+                if not type_spec_is_assignable_to(arg.type_spec(), arg_type):
                     raise TealInputError(
                         f"supplied argument {arg} at index {i} "
                         f"should have ABI typespec {arg_type} but got {arg.type_spec()}"
@@ -519,6 +522,9 @@ class ABIReturnSubroutine:
     def __init__(
         self,
         fn_implementation: Callable[..., Expr],
+        /,
+        *,
+        overriding_name: Optional[str] = None,
     ) -> None:
         self.output_kwarg_info: Optional[OutputKwArgInfo] = self._get_output_kwarg_info(
             fn_implementation
@@ -526,9 +532,17 @@ class ABIReturnSubroutine:
         self.subroutine = SubroutineDefinition(
             fn_implementation,
             return_type=TealType.none,
+            name_str=overriding_name,
             has_abi_output=self.output_kwarg_info is not None,
         )
 
+    @staticmethod
+    def name_override(name: str) -> Callable[..., "ABIReturnSubroutine"]:
+        def wrapper(fn_impl: Callable[..., Expr]) -> ABIReturnSubroutine:
+            return ABIReturnSubroutine(fn_impl, overriding_name=name)
+
+        return wrapper
+
     @classmethod
     def _get_output_kwarg_info(
         cls, fn_implementation: Callable[..., Expr]
@@ -651,22 +665,22 @@ class ABIReturnSubroutine:
                 else docstring.returns.description.replace("\n", " ").strip()
             )
 
-            # Generate the ABI method object given the subroutine args
-            # Add in description if one is set
-            for name, val in self.subroutine.annotations.items():
-                # Skip annotations for `return` and `output` in the args list
-                if name in ["return", self.OUTPUT_ARG_NAME]:
-                    continue
+        # Generate the ABI method object given the subroutine args
+        # Add in description if one is set
+        for name, val in self.subroutine.annotations.items():
+            # Skip annotations for `return` and `output` in the args list
+            if name in ["return", self.OUTPUT_ARG_NAME]:
+                continue
 
-                arg_obj = {
-                    "type": str(abi.type_spec_from_annotation(val)),
-                    "name": name,
-                }
+            arg_obj = {
+                "type": str(abi.type_spec_from_annotation(val)),
+                "name": name,
+            }
 
-                if name in arg_descs:
-                    arg_obj["desc"] = arg_descs[name]
+            if name in arg_descs:
+                arg_obj["desc"] = arg_descs[name]
 
-                args.append(arg_obj)
+            args.append(arg_obj)
 
         # Create the return obj for the method, adding description if set
         return_obj = {"type": str(self.type_of())}
