@@ -1415,10 +1415,16 @@ def test_docstring_parsing_with_different_format():
     :returns: {return_doc}
     """
     mspec_dict = ABIReturnSubroutine(documented_method).method_spec().dictify()
-    assert "descr" not in mspec_dict
+    assert "desc" not in mspec_dict
     assert mspec_dict["args"][0]["desc"] == a_doc
     assert mspec_dict["returns"]["desc"] == return_doc
 
+    # No doc
+    documented_method.__doc__ = None
+    mspec_dict = ABIReturnSubroutine(documented_method).method_spec().dictify()
+    assert "desc" not in mspec_dict
+    assert len(mspec_dict["args"]) == 1
+
     algobank_example = """Withdraw an amount of Algos held by this app.
 
     The sender of this method call will be the source of the Algos, and the destination will be
@@ -1458,3 +1464,44 @@ def test_docstring_parsing_with_different_format():
         == "An account who will receive the withdrawn Algos. This may or may not be the same as the method call sender."
     )
     assert "desc" not in mspec_dict["returns"]
+
+
+def test_relaxed_abi_arg():
+    @pt.Subroutine(pt.TealType.none)
+    def txn_checker(t: pt.abi.Transaction):
+        return pt.Seq(
+            pt.Assert(
+                pt.Or(
+                    t.get().type_enum() == pt.TxnType.Payment,
+                    t.get().type_enum() == pt.TxnType.AssetTransfer,
+                )
+            )
+        )
+
+    # Pass a payment transaction to the subroutine that expects a Transaction type
+    # this will fail if we don't have a relaxed type check for the transaction type
+    program = pt.Seq(
+        (p := pt.abi.PaymentTransaction())._set_index(pt.Txn.group_index() - pt.Int(1)),
+        txn_checker(p),
+        pt.Int(1),
+    )
+
+    pt.compileTeal(program, mode=pt.Mode.Application, version=7)
+
+
+def test_override_abi_method_name():
+    def abi_meth(a: pt.abi.Uint64, b: pt.abi.Uint64, *, output: pt.abi.Uint64):
+        return output.set(a.get() + b.get())
+
+    mspec = ABIReturnSubroutine(abi_meth).method_spec().dictify()
+    assert mspec["name"] == "abi_meth"
+
+    mspec = ABIReturnSubroutine(abi_meth, overriding_name="add").method_spec().dictify()
+    assert mspec["name"] == "add"
+
+    @ABIReturnSubroutine.name_override("overriden_add")
+    def abi_meth_2(a: pt.abi.Uint64, b: pt.abi.Uint64, *, output: pt.abi.Uint64):
+        return output.set(a.get() + b.get())
+
+    mspec = abi_meth_2.method_spec().dictify()
+    assert mspec["name"] == "overriden_add"
